package com.galaxis.manatee.entity.chuanyun.data.object;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/27 1:05 下午
 */
@Slf4j
@EqualsAndHashCode(callSuper = true)
@Data
@Entity(name = "CHUANYUN_REIMBURSEMENT")
@JsonIgnoreProperties(ignoreUnknown = true)
public class ChuanyunReimbursementDO extends BasicDO{

    /**
     * 表名
     */
    public static final String SCHEMA_CODE ="D0017894ca6679fadce4b3e9641b3c78db6890c";

    /**
     * 公司编码
     * TODO 根据beneficiaryId替换为公司编码  完成待验证
     *
     */
    private String reimburseUserCompanyCode;

    /**
     * 对公/对私
     */
    @JsonProperty("F0000063")
    private String privateOrPublic;

    /**
     * 流水号
     */
    @JsonProperty("SeqNo")
    @Column(nullable = false)
    private String serialNumber;

    /**
     * 业务日期
     */
    @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    @DateTimeFormat(pattern = "yyyy/MM/dd HH:mm:ss")
    private LocalDateTime businessDate;

    /**
     * 币种
     */
    private String currency;

    /**
     * 汇率
     */
    private BigDecimal exchangeRate;

    /**
     * 供应商ID
     */
    @JsonProperty("F0000064")
    private String supplierId;

    /**
     * 供应商编码
     * TODO 根据supplierId转化为code
     */
    private String supplierCode;

    /**
     * 项目
     */
    @JsonProperty("F0000093")
    private String groupProjectId;

    /**
     * 集团项目编码
     * TODO 根据groupProjectId转化为code
     */
    private String groupProjectCode;

    /**
     * 报销人
     */
    @JsonProperty("F0000001")
    private String reimburseUserId;

    /**
     * 报销人工号
     * TODO 根据reimburseUserId转化为code
     */
    private String reimburseUserCode;

    /**
     * 报销人成本中心编码
     * TODO 从todo 从reimburseUserCompanyId获取
     */
    private String reimburseUserCostCode;

    /**
     * 打印项目号
     */
    @JsonProperty("F0000109")
    private String printingProjectCode;

    /**
     * 打印项目类型
     */
    @JsonProperty("F0000110")
    private String printingProjectType;

    /**
     * 打印项目名称
     */
    @JsonProperty("F0000108")
    private String printingProjectName;

    /**
     * 核销方式
     */
    @JsonProperty("F0000054")
    private String writeOffMethod;

    /**
     * 报销总金额
     */
    @JsonProperty("F0000045")
    private BigDecimal amount;

    /**
     * 受益公司主键
     */
    @JsonProperty("F0000080")
    private String beneficiaryId;

    /**
     * 受益公司银行代码
     * TODO 根据beneficiaryId获得
     */
    private String beneficiaryBankCode;

    /**
     * 受益公司银行账号
     * TODO 根据beneficiaryId获得
     */
    private String beneficiaryBankAccount;

    /**
     * 结算方式
     */
    private String settlementMethod;

    /**
     * 报销原因
     */
    @JsonProperty("F0000084")
    private String reason;

    /**
     * u9传入标识
     */
    private Integer erpFlag;

    /**
     * 受益成本中心Id
     */
    @JsonProperty("F0000081")
    private String beneficiaryCostId;

    /**
     * 受益成本中心编码
     * todo 由beneficiaryCostId获得
     */
    private String beneficiaryCostCode;

    /**
     * 报销明细
     */
    @JsonProperty("D001789F9bd83c8c4057448aa49e5758784dbfc1")
    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true,fetch = FetchType.EAGER)
    private List<ChuanyunReimbursementDetailDO> detailList;

    /**
     * 发票明细
     */
    @JsonProperty("D001789Fc021e038305e4f56923a3e53e1a46dfc")
    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
    private List<ChuanyunInvoiceDO> invoiceList;


    @Transient
    @JsonProperty("F0000001Object")
    private ChuanyunObject reimburseUserObject;
}
