package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.capsule.dto.PageDTO;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunPurCostDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/27 5:14 下午
 */
@Service
@Slf4j
public class ChuanyunBasicDataScheduledTask {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunUserDao chuanyunUserDao;
    private final ChuanyunSupplierDao chuanyunSupplierDao;
    private final ChuanyunCompanyDao chuanyunCompanyDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunCostDao chuanyunCostDao;
    private final ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao;
    private final ChuanyunProjectTypeDao chuanyunProjectTypeDao;
    private final ChuanyunSpecialProcedureDao chuanyunSpecialProcedureDao;
    private final ChuanyunCostCollectionDao chuanyunCostCollectionDao;
    private final ChuanyunReimbursementItemsDao chuanyunReimbursementItemsDao;
    private final ClawFeign clawFeign;
    private final ChuanyunPurCostDao chuanyunPurCostDao;

    public ChuanyunBasicDataScheduledTask(ChuanYunManager chuanYunManager, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunUserDao chuanyunUserDao, ChuanyunSupplierDao chuanyunSupplierDao, ChuanyunCompanyDao chuanyunCompanyDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunCostDao chuanyunCostDao, ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao, ChuanyunProjectTypeDao chuanyunProjectTypeDao, ChuanyunSpecialProcedureDao chuanyunSpecialProcedureDao, ChuanyunCostCollectionDao chuanyunCostCollectionDao, ChuanyunReimbursementItemsDao chuanyunReimbursementItemsDao, ClawFeign clawFeign, ChuanyunPurCostDao chuanyunPurCostDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunUserDao = chuanyunUserDao;
        this.chuanyunSupplierDao = chuanyunSupplierDao;
        this.chuanyunCompanyDao = chuanyunCompanyDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunCostDao = chuanyunCostDao;
        this.chuanyunReimbursementSubjectDao = chuanyunReimbursementSubjectDao;
        this.chuanyunProjectTypeDao = chuanyunProjectTypeDao;
        this.chuanyunSpecialProcedureDao = chuanyunSpecialProcedureDao;
        this.chuanyunCostCollectionDao = chuanyunCostCollectionDao;
        this.chuanyunReimbursementItemsDao = chuanyunReimbursementItemsDao;
        this.clawFeign = clawFeign;
        this.chuanyunPurCostDao = chuanyunPurCostDao;
    }

    public void getReimbursementItemsManually() {
        this.getReimbursementItems();
    }

    public void getUserCompanyManually(){
        this.getUserCompany();
    }

    public void getUserListManually(){
        this.getUserList();
    }


    public void getCompanyManually(){
        this.getCompany();
    }

    public void getGroupProjectManually(){
        this.getGroupProject();
    }

    public void getCostCollectionManually() {this.getCostCollection();}

    public void getGroupProjectForSynchronizationManually() {this.getGroupProjectForSynchronization();}
    /**
     * 定时将氚云中的数据保存到manatee
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getDataFromChuanyun() {
//        savePurCost();
        //专项费用分摊
        getSpecialProcedure();
        //获取人员-归属公司信息
        getUserCompany();
        //人员报销特殊项目
        getReimbursementItems();
        //人工成本特殊项目
        getCostCollection();
        //获取集团项目数据
        getGroupProject();
        //支出档案类型
        getProjectType();
        //获取报销科目
        getReimbursementSubject();
        //获取成本中心
        getCost();
        //获取用户数据
        getUserList();
        //获取公司信息
        getCompany();
        //获取供应商
        getSupplier();

    }

    /**
     * 获取用户工号
     */
    @Scheduled(fixedDelay = 1800000L)
    private void getUserEmployeeNumber(){
        chuanyunUserDao.findAll().forEach(chuanyunUserDO -> {
            if(StringUtils.isEmpty(chuanyunUserDO.getEmployeeNumber())){
                ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),"");
            }else{
                ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),chuanyunUserDO.getEmployeeNumber());
            }
        });
        log.info("用户工号MAP更新完成");
    }

    /**
     * 从氚云中获取用户列表
     */
    private void getUserList() {
        var objectMapper=new ObjectMapper();
        var chuanyunFindAllDTO=chuanYunManager.findAll(ChuanyunUserDO.APP_CODE,ChuanyunUserDO.ACTION_NAME,ChuanyunUserDO.CONTROLLER);
        var totalCount=0;
        try {
            List<ChuanyunUserDO> userList=objectMapper.readValue(chuanyunFindAllDTO.getReturnData().getData(),new TypeReference<>(){});
            totalCount=chuanyunUserDao.saveAll(userList).size();
        } catch (Exception e) {
            e.printStackTrace();
        }
        log.info("用户信息保存保存"+totalCount+"工号映射更新完成");
    }

    /**
     * 从氚云获取集团项目信息
     */
    private void getGroupProject(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;

        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {
                    var tmpGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunGroupProjectDO.getObjectId());
                    tmpGroupProjectDO.ifPresent(projectDO -> {
                        chuanyunGroupProjectDO.setIsCreate(projectDO.getIsCreate());
                        chuanyunGroupProjectDao.delete(tmpGroupProjectDO.get());
                    });

//                    updateProjectDetail(chuanyunGroupProjectDO);
                    //处理项目对象中的项目成员数据
//                    projectMemberListCheck(chuanyunGroupProjectDO);
                    //处理项目对象中的项目经理、项目负责人数据
                    projectManagerCheck(chuanyunGroupProjectDO);
                    chuanyunGroupProjectDao.save(chuanyunGroupProjectDO);
                });
//                chuanyunGroupProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("集团项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    private void updateProjectDetail(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        var objectMapper=new ObjectMapper();
        List<ChuanyunGroupProjectDetailDO> list = new ArrayList<>();
        if (chuanyunGroupProjectDO.getDetailList()!=null&&chuanyunGroupProjectDO.getDetailList().size()>0) {
            for (ChuanyunGroupProjectDetailDO chuanyunGroupProjectDetailDO : chuanyunGroupProjectDO.getDetailList()) {
                if (!chuanyunGroupProjectDO.getProjectType().equals(chuanyunGroupProjectDetailDO.getType())) {
                    list.add(chuanyunGroupProjectDetailDO);
                }
            }
        }
        chuanyunGroupProjectDO.setDetailList(list);
        var groupProjectDetailDTO = new GroupProjectDetailDTO(chuanyunGroupProjectDO);
        String groupProjectUpdateString= null;
        try {
            groupProjectUpdateString = objectMapper.writeValueAsString(groupProjectDetailDTO);
            ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
            if (!chuanyunSaveDTO.getSuccessful()) {
                log.info(chuanyunGroupProjectDO.getObjectId()+"更新项目组织信息失败"+groupProjectUpdateString);
            }
        } catch (JsonProcessingException | BigSizeException e) {
            e.printStackTrace();
        }

    }
    /**
     * 更新项目成员方法
     * @param chuanyunGroupProjectDO    项目对象
     */
    private void projectMemberListCheck(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        var objectMapper=new ObjectMapper();
        //第一步，清除项目负责人、项目经理数据。保证能够触发权限修改流程
        GroupProjectMemberListUpdateDTO groupProjectMemberListUpdateDTO=new GroupProjectMemberListUpdateDTO(chuanyunGroupProjectDO);
        if(groupProjectMemberListUpdateDTO.getProjectMemberList()!=null&&groupProjectMemberListUpdateDTO.getProjectMemberList().length>0){
            groupProjectMemberListUpdateDTO.add();
            try {
                chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),objectMapper.writeValueAsString(groupProjectMemberListUpdateDTO));
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目成员失败"+e.getMessage());
            }
            groupProjectMemberListUpdateDTO.delete();
            //保证项目经理信息更新成功
            boolean updateFlag=false;
            while (!updateFlag){
                try {
                    var groupProjectMemberListUpdateString=objectMapper.writeValueAsString(groupProjectMemberListUpdateDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectMemberListUpdateString);
                    if(chuanyunSaveDTO.getSuccessful()){
                        updateFlag=true;
                    }else{
                        log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目成员失败"+groupProjectMemberListUpdateString);
                        log.warn(chuanyunGroupProjectDO+"");
                        Thread.sleep(5000L);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目成员失败"+e.getMessage());
                }
            }
        }
    }

    /**
     * 通过更新集团项目中的项目负责人、EBG项目经理、IBG项目经理
     * @param chuanyunGroupProjectDO    项目对象
     */
    private void projectManagerCheck(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        //将projectManger和projectOwner替换为主键
        if(!StringUtils.isEmpty(chuanyunGroupProjectDO.getProjectOwner())){
            chuanyunGroupProjectDO.setProjectOwner(chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId());
        }
        if (!StringUtils.isEmpty(chuanyunGroupProjectDO.getEbgProjectManager())){
            chuanyunGroupProjectDO.setEbgProjectManager(chuanyunGroupProjectDO.getEbgProjectManagerObject().getObjectId());
        }
        if(!StringUtils.isEmpty(chuanyunGroupProjectDO.getIbgProjectManager())){
            chuanyunGroupProjectDO.setIbgProjectManager(chuanyunGroupProjectDO.getIbgProjectManagerObject().getObjectId());
        }
        if(StringUtils.isEmpty(chuanyunGroupProjectDO.getOutProjectName())) {
            chuanyunGroupProjectDO.setOutProjectName(chuanyunGroupProjectDO.getProjectName());
        }
        var objectMapper=new ObjectMapper();
        //第一步，清除项目负责人、项目经理数据。保证能够触发权限修改流程
        GroupProjectUpdateDTO groupProjectUpdateDTO=new GroupProjectUpdateDTO();
        try {
            chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),objectMapper.writeValueAsString(groupProjectUpdateDTO));
        } catch (Exception e) {
            e.printStackTrace();
            log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+e.getMessage());
        }
        groupProjectUpdateDTO=new GroupProjectUpdateDTO(chuanyunGroupProjectDO);
        //保证项目经理信息更新成功
        boolean updateFlag=false;
        while (!updateFlag){
            try {
                var groupProjectUpdateString=objectMapper.writeValueAsString(groupProjectUpdateDTO);
                ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
                if(chuanyunSaveDTO.getSuccessful()){
                    updateFlag=true;
                    //更新成功之后重新获取集团项目组信息，防止id为空
                    var chuanyunFindResponse =  chuanYunManager.find(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId());
                    ChuanyunGroupProjectDO tmpChuanyunGroupProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                    chuanyunGroupProjectDO.setDetailList(tmpChuanyunGroupProjectDO.getDetailList());
                }else{
                    log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+groupProjectUpdateString);
                    log.warn(chuanyunGroupProjectDO+"");
                    Thread.sleep(5000L);
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+e.getMessage());
            }
        }
    }

    private void savePurCost() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunPurCostDTO> pageInfo = clawFeign.findChuanyunPurCost(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunPurCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunPurCostDTO -> {
                try {
                    PurCostDTO purCostDTO = new PurCostDTO(chuanyunPurCostDTO);
                    var purcostString=objectMapper.writeValueAsString(purCostDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.save(ChuanyunPurCostDO.SCHEMA_CODE,purcostString,true);
                    if(chuanyunSaveDTO.getSuccessful()){
                        log.info("新增项目物料成本成功");

                    }else{
                        log.warn("新增项目物料成本失败");
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 获取供应商信息
     */
    private void getSupplier(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSupplierDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSupplierDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSupplierDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("供应商信息保存保存"+totalCount);
    }

    /**
     * 更新支出档案类型
     */
    public void getProjectType() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectTypeDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunProjectTypeDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunProjectTypeDao.saveAll(result);
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("支出档案类型"+totalCount);
        }
    }

    /**
     * 更新公司信息
     */
    private void getCompany(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCompanyDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCompanyDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCompanyDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("公司信息保存保存"+totalCount);
    }

    /**
     * 更新成本中心
     */
    private void getCost(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCostDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("成本中心保存保存"+totalCount);
    }

    /**
     * 获取专项分摊规则
     */
    private void getSpecialProcedure(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSpecialProcedureDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSpecialProcedureDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSpecialProcedureDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("专项分摊信息保存保存"+totalCount);
    }

    /**
     * 获取人员报销特殊项目
     */
    private void getReimbursementItems() {
        chuanyunReimbursementItemsDao.deleteAll();
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunReimbursementItemsDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunReimbursementItemsDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunReimbursementItemsDO->{
                    //同步外部项目名称
                    var chuanyunFindResponse =  chuanYunManager.find(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunReimbursementItemsDO.getAdjustProjectId());
                    ChuanyunGroupProjectDO tmpChuanyunGroupProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                    if (!chuanyunReimbursementItemsDO.getAdjustProjectName().equals(tmpChuanyunGroupProjectDO.getOutProjectName())) {
                        chuanyunReimbursementItemsDO.setAdjustProjectName(tmpChuanyunGroupProjectDO.getOutProjectName());
                        var reimbursementItemsDTO = new ReimbursementItemsDTO(chuanyunReimbursementItemsDO);
                        String groupProjectUpdateString= null;
                        try {
                            groupProjectUpdateString = objectMapper.writeValueAsString(reimbursementItemsDTO);
                            ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunReimbursementItemsDO.SCHEMA_CODE,chuanyunReimbursementItemsDO.getObjectId(),groupProjectUpdateString);
                            if(chuanyunSaveDTO.getSuccessful()) {
                                log.info(chuanyunReimbursementItemsDO.getObjectId() + "同步外部项目名称成功" + groupProjectUpdateString);
                            }else{
                                log.warn(chuanyunReimbursementItemsDO.getObjectId()+"同步外部项目名称失败"+groupProjectUpdateString);
                            }
                        } catch (JsonProcessingException | BigSizeException e) {
                            e.printStackTrace();
                        }
                    }
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunReimbursementItemsDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人员报销特殊项目保存保存"+totalCount);
    }

    /**
     * 获取人工成本特殊项目
     */
    private void getCostCollection(){
        chuanyunCostCollectionDao.deleteAll();
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCostCollectionDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCostCollectionDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunCostCollectionDO->{
                    //同步外部项目名称
                    var chuanyunFindResponse =  chuanYunManager.find(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunCostCollectionDO.getProjectId());
                    ChuanyunGroupProjectDO tmpChuanyunGroupProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                    //外部名称有修改就更新
                    if (!tmpChuanyunGroupProjectDO.getOutProjectName().equals(chuanyunCostCollectionDO.getOutProjectName())) {
                        chuanyunCostCollectionDO.setOutProjectName(tmpChuanyunGroupProjectDO.getOutProjectName());
                        var costCollectionDTO = new CostCollectionDTO(chuanyunCostCollectionDO);
                        String groupProjectUpdateString= null;
                        try {
                            groupProjectUpdateString = objectMapper.writeValueAsString(costCollectionDTO);
                            ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunCostCollectionDO.SCHEMA_CODE,chuanyunCostCollectionDO.getObjectId(),groupProjectUpdateString);
                            if(chuanyunSaveDTO.getSuccessful()) {
                                log.info(chuanyunCostCollectionDO.getObjectId() + "同步外部项目名称成功" + groupProjectUpdateString);
                            }else{
                                log.warn(chuanyunCostCollectionDO.getObjectId()+"同步外部项目名称失败"+groupProjectUpdateString);
                            }
                        } catch (JsonProcessingException | BigSizeException e) {
                            e.printStackTrace();
                        }
                    }
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCostCollectionDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人工成本特殊项目保存保存"+totalCount);
    }

    /**
     * 获取人员-归属公司信息
     */
    private void getUserCompany(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunUserCompanyDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunUserCompanyDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunUserCompanyDO -> {
                    try {
                        if(chuanyunUserCompanyDO.getUser()!=null){
                            chuanyunUserCompanyDO.setUserId(chuanyunUserCompanyDO.getUser().getObjectId());
                        }
                        if (!StringUtils.isEmpty(chuanyunUserCompanyDO.getDepartment())) {
                            chuanyunUserCompanyDO.setDepartmentId(chuanyunUserCompanyDO.getDepartmentObject().getObjectId());
                        }
                    }catch (Exception exception){
                        log.warn(chuanyunUserCompanyDO+"");
                        log.warn(exception.getMessage());
                    }
                    try{
                        ChuanYunConstant.USER_BUSINESS_GROUP.put(chuanyunUserCompanyDO.getUserId(),chuanyunCompanyDao.findByObjectId(chuanyunUserCompanyDO.getCompanyId()).getBusinessGroup());
                    }catch (Exception e){
                        log.warn("更新用户BG信息时发生异常"+e.getMessage());
                    }
                    ChuanYunConstant.USER_DEPARTMENT.put(chuanyunUserCompanyDO.getUserId(),chuanyunUserCompanyDO.getDepartmentObject().getObjectId());
                });
                chuanyunUserCompanyDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人员归属公司信息保存保存"+totalCount);
    }

    /**
     * 获取报销科目
     */
    private void getReimbursementSubject(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunReimbursementSubjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunReimbursementSubjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunReimbursementSubjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("报销科目信息保存保存"+totalCount);
    }

    private void getGroupProjectForSynchronization(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;

        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {
                    var tmpGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunGroupProjectDO.getObjectId());
                    tmpGroupProjectDO.ifPresent(projectDO -> {
                        chuanyunGroupProjectDO.setIsCreate(projectDO.getIsCreate());
                        chuanyunGroupProjectDao.delete(tmpGroupProjectDO.get());
                    });
                });
                chuanyunGroupProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("集团项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    @Data
    class GroupProjectDetailDTO {

        @JsonProperty("D001789Fvspc68z01idftf5nqt66pkuc0")
        private List<ChuanyunGroupProjectDetailDO> detailList;

        public GroupProjectDetailDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if (null!=chuanyunGroupProjectDO) {
                if (null!=chuanyunGroupProjectDO.getDetailList()) {
                    this.detailList = chuanyunGroupProjectDO.getDetailList();
                }
            }
        }

    }

    @Data
    class CostCollectionDTO {
        /**
         * 外部项目名称
         */
        @JsonProperty("F0000008")
        public String outProjectName;

        public CostCollectionDTO(ChuanyunCostCollectionDO chuanyunCostCollectionDO) {
            if (null!=chuanyunCostCollectionDO) {
                if (null!=chuanyunCostCollectionDO.getOutProjectName()) {
                    this.outProjectName = chuanyunCostCollectionDO.getOutProjectName();
                }
            }
        }
    }

    @Data
    class ReimbursementItemsDTO {
        /**
         * 调整项目名称
         */
        @JsonProperty("F0000007")
        private String adjustProjectName;

        public ReimbursementItemsDTO(ChuanyunReimbursementItemsDO chuanyunReimbursementItemsDO) {
            if (null!=chuanyunReimbursementItemsDO) {
                if (null!=chuanyunReimbursementItemsDO.getAdjustProjectName()) {
                    this.adjustProjectName = chuanyunReimbursementItemsDO.getAdjustProjectName();
                }
            }
        }
    }

    @Data
    class PurCostDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 采购成本
         */
        @JsonProperty("F0000002")
        private BigDecimal purcost;

        /**
         * 领料成本
         */
        @JsonProperty("F0000003")
        private BigDecimal issuedcost;

        public PurCostDTO(ChuanyunPurCostDTO chuanyunPurCostDTO) {
            if (null!=chuanyunPurCostDTO) {
                if (null != chuanyunPurCostDTO.getProjectcode()) {
                    this.projectcode = chuanyunPurCostDTO.getProjectcode();
                }
                if (null != chuanyunPurCostDTO.getPurcost()) {
                    this.purcost = chuanyunPurCostDTO.getPurcost();
                }
                if (null != chuanyunPurCostDTO.getIssuedcost()) {
                    this.issuedcost = chuanyunPurCostDTO.getIssuedcost();
                }
            }
        }
    }

    /**
     * 用于刷新项目经理权限的对象
     */
    @Data
    class GroupProjectUpdateDTO{
        /**
         * 项目总监
         * U9项目责任人
         */
        @JsonProperty(value = "F0000006")
        private String projectOwner;

        /**
         * IBG项目经理
         */
        @JsonProperty("F0000012")
        private String ibgProjectManager;

        /**
         * EBG项目经理
         */
        @JsonProperty("F0000011")
        private String ebgProjectManager;

        /**
         * 外部项目名称
         */
        @JsonProperty(value="F0000038")
        private String outProjectName;


        public GroupProjectUpdateDTO() {
        }

        public GroupProjectUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectOwnerObject()){
                    this.projectOwner=chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getIbgProjectManagerObject()){
                    this.ibgProjectManager=chuanyunGroupProjectDO.getIbgProjectManagerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getEbgProjectManagerObject()){
                    this.ebgProjectManager=chuanyunGroupProjectDO.getEbgProjectManagerObject().getObjectId();
                }
                if (null!=chuanyunGroupProjectDO.getOutProjectName()) {
                    this.outProjectName = chuanyunGroupProjectDO.getOutProjectName();
                }
            }
        }
    }

    @Data
    static class GroupProjectMemberListUpdateDTO{
        /**
         * 项目组成员（前端传数据用）
         */
        @JsonProperty("F0000034")
        private String[] projectMemberList;

        public void add(){
            String[] newProjectMemberList=new String[projectMemberList.length+1];
            System.arraycopy(projectMemberList,0,newProjectMemberList,0,projectMemberList.length);
            newProjectMemberList[projectMemberList.length]="8bbb21c4-1140-4bfa-8713-6ee50ea34df0";
            projectMemberList=newProjectMemberList;
        }

        public void delete(){
            String[] newProjectMemberList=new String[projectMemberList.length-1];
            System.arraycopy(projectMemberList,0,newProjectMemberList,0,projectMemberList.length-1);
            projectMemberList=newProjectMemberList;
        }

        public GroupProjectMemberListUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectMemberList()){
                    List<String> memberList = new ArrayList<>();
                    chuanyunGroupProjectDO.getProjectMemberObject().forEach(chuanyunObject -> memberList.add(chuanyunObject.getObjectId()));
                    this.projectMemberList = memberList.toArray(new String[0]);
                }
            }
        }
    }
}
