package com.galaxis.manatee.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunClientDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunClientDO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/4 8:37 上午
 */
@Slf4j
@Service
public class ChuanyunCrmScheduledTask {
    private final Integer pageSize=20;
    private final ChuanyunClientDao chuanyunClientDao;
    private final ChuanYunManager chuanYunManager;

    public ChuanyunCrmScheduledTask(ChuanyunClientDao chuanyunClientDao, ChuanYunManager chuanYunManager) {
        this.chuanyunClientDao = chuanyunClientDao;
        this.chuanYunManager = chuanYunManager;
    }

    /**
     * 同步客户信息
     */
    @Scheduled(fixedDelay = 1800000L)
    private void clientInformationSync(){
        getClient();
    }

    /**
     * 获取供应商信息
     */
    private void getClient(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunClientDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunClientDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunClientDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("客户信息保存保存"+totalCount);
    }
}
