package com.galaxis.manatee.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunPaymentReceiveDao;
import com.galaxis.manatee.dao.ChuanyunSaleContractDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunPaymentReceiveDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSaleContractDO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/1 11:37 上午
 */
@Slf4j
@Service
public class ChuanyunProjectScheduledTask {

    private final Integer pageSize=20; final ObjectMapper objectMapper=new ObjectMapper();
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunSaleContractDao chuanyunSaleContractDao;
    private final ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao;

    public ChuanyunProjectScheduledTask(ChuanYunManager chuanYunManager, ChuanyunSaleContractDao chuanyunSaleContractDao, ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunSaleContractDao = chuanyunSaleContractDao;
        this.chuanyunPaymentReceiveDao = chuanyunPaymentReceiveDao;
    }

    /**
     * 更新销售合同数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSaleContractDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSaleContractDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSaleContractDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同信息保存保存"+totalCount);
    }

    /**
     * 更新销售合同回款数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractPaymentReceiveFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunPaymentReceiveDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunPaymentReceiveDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPaymentReceiveDO -> {
                    var tmp=chuanyunPaymentReceiveDao.findById(chuanyunPaymentReceiveDO.getObjectId());
                    tmp.ifPresent(chuanyunPaymentReceiveDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPaymentReceiveDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同回款信息保存保存"+totalCount);
    }
}
