package com.galaxis.manatee.task;

import com.galaxis.capsule.dto.ChuanyunReimbursementDTO;
import com.galaxis.manatee.dao.ChuanyunProjectDao;
import com.galaxis.manatee.dao.ChuanyunReimbursementDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunReimbursementDO;
import com.galaxis.manatee.manager.ClawFeign;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/28 7:27 下午
 */
@Service
@Slf4j
public class U9ScheduledTask {

    private final Integer pageSize=20;
    private final ChuanyunProjectDao chuanyunProjectDao;
    private final ChuanyunReimbursementDao chuanyunReimbursementDao;
    private final ClawFeign clawFeign;

    public U9ScheduledTask(ChuanyunProjectDao chuanyunProjectDao, ChuanyunReimbursementDao chuanyunReimbursementDao, ClawFeign clawFeign) {
        this.chuanyunProjectDao = chuanyunProjectDao;
        this.chuanyunReimbursementDao = chuanyunReimbursementDao;
        this.clawFeign = clawFeign;
    }

    /**
     * 定时将manatee中的信息同步到claw中
     * @deprecated U9自动从mysql中读取，不需要写到claw服务了
     */
    @Deprecated
    private void projectInformationSync(){
        putProjectToClaw();
    }

    /**
     *
     * @deprecated U9自动从mysql中读取，不需要写到claw服务了
     */
    @Deprecated
    private void reimbursementInformationSync(){
        putReimbursementToClaw();
    }

    private void putReimbursementToClaw(){
        //定时将isCreate为空的信息改为false
        var page=0;
        var totalPages=Integer.MAX_VALUE;

        while (page<totalPages){
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunReimbursementPage=chuanyunReimbursementDao.findAllByErpFlag(null,pageable);
            totalPages=chuanyunReimbursementPage.getTotalPages();
            chuanyunReimbursementPage.getContent().forEach(data->data.setErpFlag(1));
            chuanyunReimbursementDao.saveAll(chuanyunReimbursementPage.getContent());
            page++;
        }

        //同步项目数据
        page=0;
        totalPages=Integer.MAX_VALUE;
        while (page<totalPages){
            var pageable=PageRequest.of(page,pageSize);
            var chuanyunReimbursementPage=chuanyunReimbursementDao.findAllByErpFlag(0,pageable);
            chuanyunReimbursementPage.getContent().forEach(chuanyunReimbursementDO -> {
                try{
                    sendReimbursement(chuanyunReimbursementDO);
                }catch (Exception exception){
                    exception.printStackTrace();
                    log.error("保存到claw失败"+chuanyunReimbursementDO);
                }
            });
            page++;
        }
    }

    /**
     * 将项目信息同步更新到U9系统中
     */
    private void putProjectToClaw(){
        //定时将isCreate为空的信息改为false
        var page=0;
        var totalPages=Integer.MAX_VALUE;

        while (page<totalPages){
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunProjectPage=chuanyunProjectDao.findAllByIsCreate(null,pageable);
            totalPages=chuanyunProjectPage.getTotalPages();
            chuanyunProjectPage.getContent().forEach(data->data.setIsCreate(false));
            chuanyunProjectDao.saveAll(chuanyunProjectPage.getContent());
            page++;
        }

        //同步项目数据
        page=0;
        totalPages=Integer.MAX_VALUE;
        while (page<totalPages){
            var pageable=PageRequest.of(page,pageSize);
            var chuanyunProjectPage=chuanyunProjectDao.findAllByIsCreate(false,pageable);
            chuanyunProjectPage.getContent().forEach(chuanyunProjectDO -> {
                try{
                    sendProject(chuanyunProjectDO);
                }catch (Exception exception){
                    log.error("保存到claw失败"+chuanyunProjectDO.toString());
                }
            });
            page++;
        }
    }

    /**
     * 调用feign发送数据
     * @param chuanyunProjectDO 项目数据
     */
    @Transactional(rollbackFor = Exception.class)
    protected void sendProject(ChuanyunProjectDO chuanyunProjectDO){
        var responseEntity=clawFeign.saveChuanyunProject(ChuanyunProjectDO.toChuanyunProjectDTO(chuanyunProjectDO));
        if(responseEntity.getStatusCode().equals(HttpStatus.OK)){
            //如果已经是新写入claw服务需要修改是否为创建新数据的状态
            if (!chuanyunProjectDO.getIsCreate()){
                chuanyunProjectDO.setIsCreate(true);
                chuanyunProjectDao.save(chuanyunProjectDO);
            }
        }
    }

    protected void sendReimbursement(ChuanyunReimbursementDO chuanyunReimbursementDO){
        ChuanyunReimbursementDTO chuanyunReimbursementDTO=new ChuanyunReimbursementDTO();
        BeanUtils.copyProperties(chuanyunReimbursementDO,chuanyunReimbursementDTO);
        var responseEntity=clawFeign.saveReimbursement(chuanyunReimbursementDTO);
        if(responseEntity.getStatusCode().equals(HttpStatus.OK)){
            //如果已经是新写入claw服务需要修改是否为创建新数据的状态
            if (0==chuanyunReimbursementDO.getErpFlag()){
                chuanyunReimbursementDO.setErpFlag(1);
                chuanyunReimbursementDao.save(chuanyunReimbursementDO);
            }
        }
    }
}
