package com.galaxis.manatee.util;

import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import lombok.extern.slf4j.Slf4j;

import java.util.regex.Pattern;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/3 5:08 下午
 */
@Slf4j
public class AliSmsUtil {

    /**
     * 手机号校验正则表达式
     */
    private static final Pattern MOBILE_PATTERN =Pattern.compile("^[1][3,4,5,7,8][0-9]{9}$");

    /**
     * 手机号验证
     * @param  phoneNumber  手机号码
     * @return 验证通过返回true
     */
    public static boolean isMobile(final String phoneNumber) {
        return MOBILE_PATTERN.matcher(phoneNumber).matches();
    }

    /**
     * 发送注册认证短信
     * @param mobile   手机号
     * @param confirmCode   验证码
     */
    public static void sendRegisterConfirmCode(String mobile,String confirmCode,String accessKeyId,String accessSecret) throws ClientException {
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId, accessSecret);
        IAcsClient client = new DefaultAcsClient(profile);

        CommonRequest request = new CommonRequest();
        request.setMethod(MethodType.POST);
        request.setDomain("dysmsapi.aliyuncs.com");
        request.setVersion("2017-05-25");
        request.setAction("SendSms");
        request.putQueryParameter("RegionId", "cn-hangzhou");
        request.putQueryParameter("PhoneNumbers", mobile);
        request.putQueryParameter("SignName", "");
        request.putQueryParameter("TemplateCode", "");
        request.putQueryParameter("TemplateParam", "{\"code\":\""+confirmCode+"\"}");
        CommonResponse response = client.getCommonResponse(request);
        log.info(response.getData());
    }
}
